#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators
{
	public class MAB : Indicator
	{
		private EMA EMA1;
		private EMA EMA2;
		private Series<double> Dst;
		private double DV;
		private double Dev;
		
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description									= @"Moving Average Bands (MAB) was published in the August 2021 Technical Analysis of Stocks and Commodities article titled 'Moving Average Bands Part 1' by Vitali Apirine.";
				Name										= "MAB";
				Calculate									= Calculate.OnBarClose;
				IsOverlay									= false;
				DisplayInDataBox							= true;
				DrawOnPricePanel							= true;
				DrawHorizontalGridLines						= true;
				DrawVerticalGridLines						= true;
				PaintPriceMarkers							= true;
				ScaleJustification							= NinjaTrader.Gui.Chart.ScaleJustification.Right;
				//Disable this property if your indicator requires custom values that cumulate with each new market data event. 
				//See Help Guide for additional information.
				IsSuspendedWhileInactive					= true;
				Periods1									= 50;
				Periods2									= 10;
				Mltp										= 1;
				AddPlot(Brushes.Orange, "MA1");
				AddPlot(Brushes.RoyalBlue, "MA2");
				AddPlot(Brushes.Orange, "UpperMA1Band");
				AddPlot(Brushes.Orange, "LowerMA1Band");
			}
			else if (State == State.DataLoaded)
			{
				Dst = new Series<double>(this, MaximumBarsLookBack.TwoHundredFiftySix);
				EMA1 = EMA(Periods1);
				EMA2 = EMA(Periods2);
			}
		}

		protected override void OnBarUpdate()
		{
			Dst[0] = Math.Pow(EMA1[0] - EMA2[0], 2);

			DV = SUM(Dst, Periods2)[0] / Periods2;
			
			Dev = Math.Sqrt(DV) * Mltp;
			
			Values[0][0] = EMA1[0];
			Values[1][0] = EMA2[0];
			Values[2][0] = EMA1[0] + Dev;
			Values[3][0] = EMA1[0] - Dev;
				
		}

		#region Properties
		[NinjaScriptProperty]
		[Range(1, int.MaxValue)]
		[Display(Name="Slow Period", Description="Period for longer term Moving Average", Order=1, GroupName="Parameters")]
		public int Periods1
		{ get; set; }

		[NinjaScriptProperty]
		[Range(1, int.MaxValue)]
		[Display(Name="Fast Period", Description="Period for shorter term Moving Average", Order=2, GroupName="Parameters")]
		public int Periods2
		{ get; set; }
		
		[NinjaScriptProperty]
		[Range(0, double.MaxValue)]
		[Display(Name="Deviation", Description="Deviation multiplier", Order=3, GroupName="Parameters")]
		public double Mltp
		{ get; set; }

		[Browsable(false)]
		[XmlIgnore]
		public Series<double> MA1
		{
			get { return Values[0]; }
		}

		[Browsable(false)]
		[XmlIgnore]
		public Series<double> MA2
		{
			get { return Values[1]; }
		}

		[Browsable(false)]
		[XmlIgnore]
		public Series<double> UpperMA1Band
		{
			get { return Values[2]; }
		}

		[Browsable(false)]
		[XmlIgnore]
		public Series<double> LowerMA1Band
		{
			get { return Values[3]; }
		}
		#endregion

	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private MAB[] cacheMAB;
		public MAB MAB(int periods1, int periods2, double mltp)
		{
			return MAB(Input, periods1, periods2, mltp);
		}

		public MAB MAB(ISeries<double> input, int periods1, int periods2, double mltp)
		{
			if (cacheMAB != null)
				for (int idx = 0; idx < cacheMAB.Length; idx++)
					if (cacheMAB[idx] != null && cacheMAB[idx].Periods1 == periods1 && cacheMAB[idx].Periods2 == periods2 && cacheMAB[idx].Mltp == mltp && cacheMAB[idx].EqualsInput(input))
						return cacheMAB[idx];
			return CacheIndicator<MAB>(new MAB(){ Periods1 = periods1, Periods2 = periods2, Mltp = mltp }, input, ref cacheMAB);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.MAB MAB(int periods1, int periods2, double mltp)
		{
			return indicator.MAB(Input, periods1, periods2, mltp);
		}

		public Indicators.MAB MAB(ISeries<double> input , int periods1, int periods2, double mltp)
		{
			return indicator.MAB(input, periods1, periods2, mltp);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.MAB MAB(int periods1, int periods2, double mltp)
		{
			return indicator.MAB(Input, periods1, periods2, mltp);
		}

		public Indicators.MAB MAB(ISeries<double> input , int periods1, int periods2, double mltp)
		{
			return indicator.MAB(input, periods1, periods2, mltp);
		}
	}
}

#endregion
