#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators
{
	public class MABW : Indicator
	{
		private EMA EMA1;
		private EMA EMA2;
		private Series<double> Dst;
		private double DV;
		private double Dev;
		private double UpBnd;
		private double DwnBnd;
		
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description									= @"Moving Average Band Width (MABW) was published in the August 2021 Technical Analysis of Stocks and Commodities article titled 'Moving Average Band Width Part 2' by Vitali Apirine.";
				Name										= "MABW";
				Calculate									= Calculate.OnBarClose;
				IsOverlay									= false;
				DisplayInDataBox							= true;
				DrawOnPricePanel							= true;
				DrawHorizontalGridLines						= true;
				DrawVerticalGridLines						= true;
				PaintPriceMarkers							= true;
				ScaleJustification							= NinjaTrader.Gui.Chart.ScaleJustification.Right;
				//Disable this property if your indicator requires custom values that cumulate with each new market data event. 
				//See Help Guide for additional information.
				IsSuspendedWhileInactive					= true;
				Periods1									= 50;
				Periods2									= 10;
				Mltp										= 1;
				AddPlot(Brushes.Orange, "BNDW");
				AddPlot(Brushes.RoyalBlue, "LLV");
			}
			else if (State == State.DataLoaded)
			{
				Dst = new Series<double>(this, MaximumBarsLookBack.TwoHundredFiftySix);
				EMA1 = EMA(Periods1);
				EMA2 = EMA(Periods2);
			}
		}

		protected override void OnBarUpdate()
		{
			Dst[0] = Math.Pow(EMA1[0] - EMA2[0], 2);

			DV = SUM(Dst, Periods2)[0] / Periods2;
			
			Dev = Math.Sqrt(DV) * Mltp;
			
			UpBnd = EMA1[0] + Dev;
			DwnBnd = EMA1[0] - Dev;
			
			Values[0][0] = (UpBnd - DwnBnd)/EMA1[0] * 100;
			Values[1][0] = MIN(Values[0], Periods1)[0];
				
		}

		#region Properties
		[NinjaScriptProperty]
		[Range(1, int.MaxValue)]
		[Display(Name="Slow Period", Description="Period for longer term Moving Average", Order=1, GroupName="Parameters")]
		public int Periods1
		{ get; set; }

		[NinjaScriptProperty]
		[Range(1, int.MaxValue)]
		[Display(Name="Fast Period", Description="Period for shorter term Moving Average", Order=2, GroupName="Parameters")]
		public int Periods2
		{ get; set; }
		
		[NinjaScriptProperty]
		[Range(0, double.MaxValue)]
		[Display(Name="Deviation", Description="Deviation multiplier", Order=3, GroupName="Parameters")]
		public double Mltp
		{ get; set; }

		[Browsable(false)]
		[XmlIgnore]
		public Series<double> BNDW
		{
			get { return Values[0]; }
		}

		[Browsable(false)]
		[XmlIgnore]
		public Series<double> LLV
		{
			get { return Values[1]; }
		}
		#endregion

	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private MABW[] cacheMABW;
		public MABW MABW(int periods1, int periods2, double mltp)
		{
			return MABW(Input, periods1, periods2, mltp);
		}

		public MABW MABW(ISeries<double> input, int periods1, int periods2, double mltp)
		{
			if (cacheMABW != null)
				for (int idx = 0; idx < cacheMABW.Length; idx++)
					if (cacheMABW[idx] != null && cacheMABW[idx].Periods1 == periods1 && cacheMABW[idx].Periods2 == periods2 && cacheMABW[idx].Mltp == mltp && cacheMABW[idx].EqualsInput(input))
						return cacheMABW[idx];
			return CacheIndicator<MABW>(new MABW(){ Periods1 = periods1, Periods2 = periods2, Mltp = mltp }, input, ref cacheMABW);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.MABW MABW(int periods1, int periods2, double mltp)
		{
			return indicator.MABW(Input, periods1, periods2, mltp);
		}

		public Indicators.MABW MABW(ISeries<double> input , int periods1, int periods2, double mltp)
		{
			return indicator.MABW(input, periods1, periods2, mltp);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.MABW MABW(int periods1, int periods2, double mltp)
		{
			return indicator.MABW(Input, periods1, periods2, mltp);
		}

		public Indicators.MABW MABW(ISeries<double> input , int periods1, int periods2, double mltp)
		{
			return indicator.MABW(input, periods1, periods2, mltp);
		}
	}
}

#endregion
